;;;; geopoint.scm  -*- Scheme -*-
;;;; Kon Lovett, Jul '18
;;;; Kon Lovett, May '17

(module geopoint

(;export
  ;
  make-geopoint
  geopoint? check-geopoint error-geopoint
  geopoint-latitude geopoint-longitude
  ;
  geopoint-strictly-above? geopoint-above? geopoint-strictly-below? geopoint-below?
  geopoint-strictly-left? geopoint-left? geopoint-strictly-right? geopoint-right?
  ;
  geopoint= geopoint< geopoint> geopoint<= geopoint>=)

(import scheme
  (srfi 9)
  (chicken base)
  (chicken type)
  (only type-checks-basic define-check+error-type)
  (only (type-checks-numbers scheme) check-real))

(define-type geopoint (struct geopoint))

(: *make-geopoint (number number --> geopoint))
(: geopoint? (* -> boolean : geopoint))
(: *geopoint-latitude (geopoint --> number))
(: *geopoint-longitude (geopoint --> number))
(: make-geopoint (number number --> geopoint))
(: geopoint-latitude (geopoint --> number))
(: geopoint-longitude (geopoint --> number))
(: geopoint-strictly-above? (geopoint geopoint --> boolean))
(: geopoint-above? (geopoint geopoint --> boolean))
(: geopoint-strictly-below? (geopoint geopoint --> boolean))
(: geopoint-below? (geopoint geopoint --> boolean))
(: geopoint-strictly-left? (geopoint geopoint --> boolean))
(: geopoint-left? (geopoint geopoint --> boolean))
(: geopoint-strictly-right? (geopoint geopoint --> boolean))
(: geopoint-right? (geopoint geopoint --> boolean))
(: geopoint= (geopoint geopoint --> boolean))
(: geopoint< (geopoint geopoint --> boolean))
(: geopoint> (geopoint geopoint --> boolean))
(: geopoint<= (geopoint geopoint --> boolean))
(: geopoint>= (geopoint geopoint --> boolean))

;;

(include "geo-constants")

;;

(define-record-type geopoint
  (*make-geopoint lat lon)
  geopoint?
  (lat *geopoint-latitude)
  (lon *geopoint-longitude) )

(define (make-geopoint lat lon)
  (*make-geopoint
    (check-real 'make-geopoint lat 'lat)
    (check-real 'make-geopoint lon 'lon)) )

(define-check+error-type geopoint)

(define (geopoint-latitude gp)
  (*geopoint-latitude (check-geopoint 'geopoint-latitude gp)) )

(define (geopoint-longitude gp)
  (*geopoint-longitude (check-geopoint 'geopoint-longitude gp)) )

(define-record-printer (geopoint gp out)
  (display "#,(geopoint " out)
  (display (*geopoint-longitude gp) out)
  (display #\space out)
  (display (*geopoint-latitude gp) out)
  (display #\) out) )

;;

(define (geopoint-strictly-above? gp1 gp2)
  (<  (*geopoint-latitude (check-geopoint 'geopoint-strictly-above gp1))
      (*geopoint-latitude (check-geopoint 'geopoint-strictly-above gp2))) )

(define (geopoint-above? gp1 gp2)
  (<= (*geopoint-latitude (check-geopoint 'geopoint-above gp1))
      (*geopoint-latitude (check-geopoint 'geopoint-above gp2))) )

(define (geopoint-strictly-below? gp1 gp2)
  (>  (*geopoint-latitude (check-geopoint 'geopoint-strictly-below gp1))
      (*geopoint-latitude (check-geopoint 'geopoint-strictly-below gp2))) )

(define (geopoint-below? gp1 gp2)
  (>= (*geopoint-latitude (check-geopoint 'geopoint-below gp1))
      (*geopoint-latitude (check-geopoint 'geopoint-below gp2))) )

(define (geopoint-strictly-left? gp1 gp2)
  (<  (*geopoint-longitude (check-geopoint 'geopoint-strictly-left gp1))
      (*geopoint-longitude (check-geopoint 'geopoint-strictly-left gp2))) )

(define (geopoint-left? gp1 gp2)
  (<= (*geopoint-longitude (check-geopoint 'geopoint-left gp1))
      (*geopoint-longitude (check-geopoint 'geopoint-left gp2))) )

(define (geopoint-strictly-right? gp1 gp2)
  (>  (*geopoint-longitude (check-geopoint 'geopoint-strictly-right gp1))
      (*geopoint-longitude (check-geopoint 'geopoint-strictly-right gp2))) )

(define (geopoint-right? gp1 gp2)
  (>= (*geopoint-longitude (check-geopoint 'geopoint-right gp1))
      (*geopoint-longitude (check-geopoint 'geopoint-right gp2))) )

;;

(define (geopoint= gp1 gp2)
  (check-geopoint 'geopoint= gp1)
  (check-geopoint 'geopoint= gp2)
  (and  (= (*geopoint-latitude gp1) (*geopoint-latitude gp2))
        (= (*geopoint-longitude gp1) (*geopoint-longitude gp2)) ) )

(define (geopoint< gp1 gp2)
  (check-geopoint 'geopoint< gp1)
  (check-geopoint 'geopoint< gp2)
  (and  (< (*geopoint-latitude gp1) (*geopoint-latitude gp2))
        (< (*geopoint-longitude gp1) (*geopoint-longitude gp2)) ) )

(define (geopoint> gp1 gp2)
  (check-geopoint 'geopoint> gp1)
  (check-geopoint 'geopoint> gp2)
  (and  (> (*geopoint-latitude gp1) (*geopoint-latitude gp2))
        (> (*geopoint-longitude gp1) (*geopoint-longitude gp2)) ) )

(define (geopoint<= gp1 gp2)
  (check-geopoint 'geopoint<= gp1)
  (check-geopoint 'geopoint<= gp2)
  (and  (<= (*geopoint-latitude gp1) (*geopoint-latitude gp2))
        (<= (*geopoint-longitude gp1) (*geopoint-longitude gp2)) ) )

(define (geopoint>= gp1 gp2)
  (check-geopoint 'geopoint>= gp1)
  (check-geopoint 'geopoint>= gp2)
  (and  (>= (*geopoint-latitude gp1) (*geopoint-latitude gp2))
        (>= (*geopoint-longitude gp1) (*geopoint-longitude gp2)) ) )

) ;geopoint
